#
# Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
# All rights reserved.
# This component and the accompanying materials are made available
# under the terms of "Eclipse Public License v1.0"
# which accompanies this distribution, and is available
# at the URL "http://www.eclipse.org/legal/epl-v10.html".
#
# Initial Contributors:
# Nokia Corporation - initial contribution.
#
# Contributors:
#
# Description:
# Dependency processors
#

'''
Dependency processors provide a modular way to add dependency data to
PackageModel.Components.

A Packager is given dependency processor class + initarg pairs using the
addProcessor method. When it has created a Component, it iterates through this
list, initialising a processor with the corresponding initarg from each entry.
Finally, it calls the processor instance with Component, BuildData and FileMap
instances as arguments.

BuildData.dependencyData is a dictionary carrying Component-specific data for
dependency processors. The keys are processor class names.

To summarize:
    - The processor __call__() method is responsible for adding the dependency
    data.
    - The __call__() method takes Component, FileMap and BuildData as arguments
    - The processor __init__() takes a single argument. Every instance of a
    processor class is initialized using the same argument.
    - Component-specific data is passed to  dependency processors via BuildData.

Example:
    - Dependency processor I{Foo} uses config file I{/bar/baz.conf} containing
    instructions how to process the components.
    - Each Component is given unique dependency info for I{Foo} in a I{FooData}
    instance::

        pack = Packager(storage,                # create a packager
                epocRoot,
                sourceRoot)
        pack.addProcessor(Foo, "bar/baz.conf")  # specify a dependency processor
        build = PlainBuildData()                # create builddata
        build.setComponentName("myComponent")
        build.dependencyData["Foo"] = FooData("Dependency info for myComponent")
                                                # tag on data for use by Foo
        pack.addComponent(build)                # make component and package it

    - Packager I{pack} will effectively do::

        component = Component(build.getComponentName())
                                                # create Component
        processor = Foo("bar/baz.conf")         # create a Foo
        processor(component, build, filemap)    # call Foo to apply dependencies

    - I{Foo} will contain the following::

        def __init__(self, configfile):
            self.readConfig(configfile)

        def __call__(self, component, buildData, fileMap):
            uniquedata = buildData.dependencyData.get(self.__class__.__name__)
            # next use config, uniquedata, buildData and fileMap to apply
            # dependencies to component
'''